package de.fraunhofer.iem.dataprovider.kpi.strategy

import de.fraunhofer.iem.dataprovider.kpi.dto.KpiCalculationDto
import de.fraunhofer.iem.dataprovider.kpi.enumeration.KpiKind
import org.junit.jupiter.api.Test

class AggregationKPICalculationStrategyTest {

    private fun getRootKpiPartiallyEmpty(): KpiCalculationDto {
        val processComplianceKPI = KpiCalculationDto(
            kind = KpiKind.PROCESS_COMPLIANCE,
            calculationStrategy = RawValueKPICalculationStrategy(25),
            value = 25,
            isEmpty = false
        )

        val processTransparencyKPI = KpiCalculationDto(
            kind = KpiKind.PROCESS_TRANSPARENCY,
            calculationStrategy = RawValueKPICalculationStrategy(50),
            value = 50,
            isEmpty = false
        )

        val securityKPI = KpiCalculationDto(
            kind = KpiKind.SECURITY,
            calculationStrategy = AggregationKPICalculationStrategy(),
            value = -1,
            isEmpty = true
        )

        val transparencyKpi = KpiCalculationDto(
            kind = KpiKind.PROCESS_TRANSPARENCY,
            calculationStrategy = AggregationKPICalculationStrategy(),
            value = -1,
            isEmpty = true
        )

        transparencyKpi.addChildKPI(
            KpiCalculationDto(
                kind = KpiKind.PROCESS_TRANSPARENCY,
                calculationStrategy = RawValueKPICalculationStrategy(-1),
                value = -1,
                isEmpty = true
            ),
            1.0
        )

        val rootKPI = KpiCalculationDto(
            kind = KpiKind.ROOT,
            calculationStrategy = AggregationKPICalculationStrategy(),
        )

        rootKPI.addChildKPI(processTransparencyKPI, 0.25)
        rootKPI.addChildKPI(processComplianceKPI, 0.25)
        rootKPI.addChildKPI(transparencyKpi, 0.25)
        rootKPI.addChildKPI(securityKPI, 0.25)

        return rootKPI
    }

    private fun getRootKpiComplete(): KpiCalculationDto {
        val processComplianceKPI = KpiCalculationDto(
            kind = KpiKind.PROCESS_COMPLIANCE,
            calculationStrategy = RawValueKPICalculationStrategy(25),
            value = 25,
            isEmpty = false
        )

        val processTransparencyKPI = KpiCalculationDto(
            kind = KpiKind.PROCESS_TRANSPARENCY,
            calculationStrategy = RawValueKPICalculationStrategy(50),
            value = 50,
            isEmpty = false
        )

        val securityKPI = KpiCalculationDto(
            kind = KpiKind.SECURITY,
            calculationStrategy = RawValueKPICalculationStrategy(100),
            value = 100,
            isEmpty = false
        )

        val rootKPI = KpiCalculationDto(
            kind = KpiKind.ROOT,
            calculationStrategy = AggregationKPICalculationStrategy(),
        )

        rootKPI.addChildKPI(processTransparencyKPI, 0.25)
        rootKPI.addChildKPI(processComplianceKPI, 0.25)
        rootKPI.addChildKPI(securityKPI, 0.5)

        return rootKPI
    }

    @Test
    fun calculateKPI() {
        val rootKpiPartiallyEmpty = getRootKpiPartiallyEmpty()
        rootKpiPartiallyEmpty.getHierarchyEdges()

        val sum = AggregationKPICalculationStrategy().calculateKPI(rootKpiPartiallyEmpty.getHierarchyEdges())
        assert(sum.first == 37)

        val rootKpi = getRootKpiComplete()
        val sum2 = AggregationKPICalculationStrategy().calculateKPI(rootKpi.getHierarchyEdges())
        assert(sum2.first == 68)
    }
}
